import click
import typer

from .cli_helper import (
    prompt_car_name_position_and_commands,
    prompt_field_dims_and_initialize_field,
    prompt_post_simulation_menu_and_get_selection,
    prompt_pre_simulation_menu_and_get_selection,
    render_interactive_simulation,
    render_intro_text,
    render_simulation_outcome,
)
from .constants import CLI_EPILOG_TEXT, CLI_HELP_TEXT
from .simulator import Simulator

app = typer.Typer(rich_markup_mode="rich")


@app.command(help=CLI_HELP_TEXT, epilog=CLI_EPILOG_TEXT)
def main():

    while True:
        click.clear()
        render_intro_text()

        field = prompt_field_dims_and_initialize_field()
        simulator = Simulator(field)

        while True:

            selection = prompt_pre_simulation_menu_and_get_selection(simulator)
            if selection == "1":
                prompt_car_name_position_and_commands(simulator)
            else:
                if selection == "2":
                    render_simulation_outcome(simulator)
                else:
                    render_interactive_simulation(simulator)
                break

        selection = prompt_post_simulation_menu_and_get_selection()
        if selection == "1":
            continue
        else:
            break


if __name__ == "__main__":
    app()
