from dataclasses import dataclass
from typing import Self

from .enums import Direction
from .field import Field


@dataclass
class Position:
    x: int
    y: int

    def __hash__(self) -> int:
        return hash((self.x, self.y))

    def __str__(self) -> str:
        return f"({self.x}, {self.y})"

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, Position):
            return NotImplemented
        return self.x == other.x and self.y == other.y

    @classmethod
    def from_moving_forward_along_direction(
        cls, cur_position: Self, move_direction: Direction
    ) -> Self:

        next_x: int
        next_y: int

        if move_direction == Direction.NORTH:
            next_x = cur_position.x
            next_y = cur_position.y + 1

        elif move_direction == Direction.EAST:
            next_x = cur_position.x + 1
            next_y = cur_position.y

        elif move_direction == Direction.SOUTH:
            next_x = cur_position.x
            next_y = cur_position.y - 1

        else:
            next_x = cur_position.x - 1
            next_y = cur_position.y

        return cls(next_x, next_y)

    def is_within_field_bounds(self, field: Field) -> bool:
        return 0 <= self.x <= field.width - 1 and 0 <= self.y <= field.height - 1
