from dataclasses import dataclass
from typing import Literal, Self

from .enums import Command, Direction
from .field import Field
from .position import Position


@dataclass
class PositionVector:
    position: Position
    direction: Direction

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, PositionVector):
            return NotImplemented

        return self.position == other.position and self.direction == other.direction

    @staticmethod
    def get_next_direction(
        cur_direction: Direction,
        direction_command: Literal[Command.LEFT, Command.RIGHT],
    ) -> Direction:

        if direction_command == Command.RIGHT:
            if cur_direction == Direction.NORTH:
                return Direction.EAST

            if cur_direction == Direction.EAST:
                return Direction.SOUTH

            if cur_direction == Direction.SOUTH:
                return Direction.WEST

            else:
                return Direction.NORTH

        else:
            if cur_direction == Direction.NORTH:
                return Direction.WEST

            if cur_direction == Direction.WEST:
                return Direction.SOUTH

            if cur_direction == Direction.SOUTH:
                return Direction.EAST

            else:
                return Direction.NORTH

    @classmethod
    def from_applying_command_to_position_on_field(
        cls, command: Command, cur_position_vector: Self, field: Field
    ) -> Self:
        cur_position = cur_position_vector.position
        cur_direction = cur_position_vector.direction

        if command == Command.FORWARD:
            next_position = Position.from_moving_forward_along_direction(
                cur_position, cur_direction
            )

            if next_position.is_within_field_bounds(field):
                return cls(next_position, cur_direction)

            else:
                return cur_position_vector

        else:
            new_direction = PositionVector.get_next_direction(cur_direction, command)
            return cls(cur_position, new_direction)
